@@ -2,7 +2,7 @@
 
 """Azure image class module."""
 
-# Copyright (c) 2021 SUSE LLC
+# Copyright (c) 2022 SUSE LLC
 #
 # This file is part of azure_img_utils. azure_img_utils provides an
 # api and command line utilities for handling images in the Azure Cloud.
@@ -20,39 +20,42 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import jmespath
 import json
 import logging
+import lzma
 import os
 
 from azure.core.exceptions import ResourceNotFoundError
 from azure.mgmt.compute import ComputeManagementClient
 
 from azure_img_utils.auth import get_client_from_json, acquire_access_token
-from azure_img_utils.exceptions import AzureImgUtilsException
+
+from azure_img_utils.exceptions import (
+    AzureImgUtilsException,
+    AzureImgUtilsStorageException,
+)
+
+from azure_img_utils.filetype import FileType
+
 from azure_img_utils.storage import (
+    get_blob_client,
     get_blob_service,
     get_blob_url,
-    blob_exists,
-    delete_blob,
-    upload_azure_file
 )
+
 from azure_img_utils.compute import (
-    create_image,
     create_gallery_image_definition_version,
-    delete_image,
     get_image,
-    image_exists,
     remove_gallery_image_version,
     retrieve_gallery_image_version
 )
+
 from azure_img_utils.cloud_partner import (
-    get_cloud_partner_offer_status,
-    get_cloud_partner_operation,
-    request_cloud_partner_offer_doc,
     add_image_version_to_offer,
-    put_cloud_partner_offer_doc,
-    publish_cloud_partner_offer,
-    go_live_with_cloud_partner_offer,
+    get_cloud_partner_api_headers,
+    get_cloud_partner_endpoint,
+    process_request,
     remove_image_version_from_offer
 )
 
@@ -99,20 +102,23 @@ class AzureImage(object):
 
     def image_blob_exists(self, blob_name: str):
         """Return True if image blob exists in the configured container."""
-        return blob_exists(
+        blob_client = get_blob_client(
             self.blob_service_client,
             blob_name,
             self.container
         )
+        return blob_client.exists()
 
     def delete_storage_blob(self, blob_name: str):
         """Delete blob if it exists in the configured container."""
         try:
-            delete_blob(
+            blob_client = get_blob_client(
                 self.blob_service_client,
                 blob_name,
                 self.container
             )
+            blob_client.delete_blob()
+
         except ResourceNotFoundError:
             self.log.debug(
                 f'Blob {blob_name} not found. '
@@ -125,8 +131,8 @@ class AzureImage(object):
     def upload_image_blob(
         self,
         image_file: str,
-        max_workers: int = None,
-        max_retry_attempts: int = None,
+        max_workers: int = 5,
+        max_attempts: int = 5,
         blob_name: str = None,
         force_replace_image: bool = False,
         is_page_blob: bool = True,
@@ -149,36 +155,63 @@ class AzureImage(object):
         elif self.image_blob_exists(blob_name) and force_replace_image:
             self.delete_storage_blob(blob_name)
 
-        kwargs = {
-            'is_page_blob': is_page_blob,
-            'expand_image': expand_image
-        }
-
-        if max_workers:
-            kwargs['max_workers'] = max_workers
-
-        if max_retry_attempts:
-            kwargs['max_retry_attempts'] = max_retry_attempts
+        if max_attempts <= 0:
+            raise Exception(
+                f'max_attempts parameter value has to be >0, '
+                f'{max_attempts} provided.'
+            )
 
         try:
-            upload_azure_file(
-                blob_name,
-                self.container,
-                image_file,
+            blob_client = get_blob_client(
                 self.blob_service_client,
-                **kwargs
+                blob_name,
+                self.container
             )
+
+            if is_page_blob:
+                blob_type = 'PageBlob'
+            else:
+                blob_type = 'BlockBlob'
+
+            system_image_file_type = FileType(image_file)
+            if system_image_file_type.is_xz() and expand_image:
+                open_image = lzma.LZMAFile
+            else:
+                open_image = open
+
+            msg = ''
+            while max_attempts > 0:
+                with open_image(image_file, 'rb') as image_stream:
+                    try:
+                        blob_client.upload_blob(
+                            image_stream,
+                            blob_type=blob_type,
+                            length=system_image_file_type.get_size(),
+                            max_concurrency=max_workers
+                        )
+                        return blob_name
+
+                    except Exception as error:
+                        msg = error
+                        max_attempts -= 1
+
+            raise AzureImgUtilsStorageException(
+                'Unable to upload {0}: {1}'.format(image_file, msg)
+            )
+
         except FileNotFoundError:
             raise AzureImgUtilsException(
                 f'Image file {image_file} not found. Ensure the path to'
                 f' the file is correct.'
             )
 
-        return blob_name
-
     def image_exists(self, image_name: str) -> bool:
         """Return True if image exists, false otherwise."""
-        return image_exists(self.compute_client, image_name)
+        images = self.compute_client.images.list()
+        for image in images:
+            if image.name == image_name:
+                return True
+        return False
 
     def gallery_image_version_exists(
         self,
@@ -207,11 +240,11 @@ class AzureImage(object):
                 'Resource group is required to delete a compute image'
             )
 
-        delete_image(
-            self.compute_client,
+        async_delete_image = self.compute_client.images.begin_delete(
             self.resource_group,
             image_name
         )
+        async_delete_image.result()
 
     def delete_gallery_image_version(
         self,
@@ -288,6 +321,8 @@ class AzureImage(object):
 
         If image exists and force replace is True delete
         the existing image before creation.
+
+        hyper v generation of V2 is uefi and V1 is legacy bios.
         """
         if not self.container:
             raise AzureImgUtilsException(
@@ -304,30 +339,40 @@ class AzureImage(object):
                 'Storage account is required to create a compute image'
             )
 
-        exists = image_exists(self.compute_client, image_name)
+        exists = self.image_exists(image_name)
 
         if exists and force_replace_image:
-            delete_image(
-                self.compute_client,
-                self.resource_group,
-                image_name
-            )
+            self.delete_compute_image(image_name)
+
         elif exists and not force_replace_image:
             raise AzureImgUtilsException(
                 'Image already exists. To force deletion and re-create '
                 'the image use "force_replace_image=True".'
             )
 
-        return create_image(
-            blob_name,
-            image_name,
-            self.compute_client,
-            self.container,
+        async_create_image = self.compute_client.images.begin_create_or_update(
             self.resource_group,
-            self.storage_account,
-            region,
-            hyper_v_generation
+            image_name,
+            {
+                'location': region,
+                'hyper_v_generation': hyper_v_generation,
+                'storage_profile': {
+                    'os_disk': {
+                        'os_type': 'Linux',
+                        'os_state': 'Generalized',
+                        'caching': 'ReadWrite',
+                        'blob_uri': 'https://{0}.{1}/{2}/{3}'.format(
+                            self.storage_account,
+                            'blob.core.windows.net',
+                            self.container,
+                            blob_name
+                        )
+                    }
+                }
+            }
         )
+        async_create_image.result()
+        return image_name
 
     def create_gallery_image_version(
         self,
@@ -402,12 +447,20 @@ class AzureImage(object):
         """
         Return the offer doc dictionary for the given offer.
         """
-        return request_cloud_partner_offer_doc(
-            self.access_token,
+        endpoint = get_cloud_partner_endpoint(
             offer_id,
             publisher_id
         )
 
+        headers = get_cloud_partner_api_headers(self.access_token)
+
+        response = process_request(
+            endpoint,
+            headers,
+            method='get'
+        )
+        return response
+
     def upload_offer_doc(
         self,
         offer_id: str,
@@ -419,12 +472,22 @@ class AzureImage(object):
 
         offer_doc is a dictionary defining the offer details.
         """
-        put_cloud_partner_offer_doc(
-            self.access_token,
-            offer_doc,
+        endpoint = get_cloud_partner_endpoint(
             offer_id,
             publisher_id
         )
+        headers = get_cloud_partner_api_headers(
+            self.access_token,
+            content_type='application/json',
+            if_match='*'
+        )
+
+        process_request(
+            endpoint,
+            headers,
+            data=offer_doc,
+            method='put'
+        )
 
     def add_image_to_offer(
         self,
@@ -538,13 +601,27 @@ class AzureImage(object):
 
         Returns the operation uri.
         """
-        return publish_cloud_partner_offer(
-            self.access_token,
+        endpoint = get_cloud_partner_endpoint(
             offer_id,
             publisher_id,
-            notification_emails
+            publish=True
         )
 
+        headers = get_cloud_partner_api_headers(
+            self.access_token,
+            content_type='application/json'
+        )
+
+        response = process_request(
+            endpoint,
+            headers,
+            data={'metadata': {'notification-emails': notification_emails}},
+            method='post',
+            json_response=False
+        )
+
+        return response.headers['Location']
+
     def go_live_with_offer(
         self,
         offer_id: str,
@@ -557,27 +634,69 @@ class AzureImage(object):
 
         Returns the operation uri.
         """
-        return go_live_with_cloud_partner_offer(
-            self.access_token,
+        endpoint = get_cloud_partner_endpoint(
             offer_id,
-            publisher_id
+            publisher_id,
+            go_live=True
         )
+        headers = get_cloud_partner_api_headers(
+            self.access_token,
+            content_type='application/json'
+        )
+
+        response = process_request(
+            endpoint,
+            headers,
+            method='post',
+            json_response=False
+        )
+
+        return response.headers['Location']
 
     def get_offer_status(self, offer_id, publisher_id) -> str:
         """
         Returns the status of the offer.
         """
-        return get_cloud_partner_offer_status(
-            self.access_token,
+        endpoint = get_cloud_partner_endpoint(
             offer_id,
-            publisher_id
+            publisher_id,
+            status=True
         )
+        headers = get_cloud_partner_api_headers(self.access_token)
+
+        response = process_request(
+            endpoint,
+            headers,
+            method='get'
+        )
+
+        status = response.get('status', 'unkown')
+
+        if status == 'running':
+            signoff_status = jmespath.search(
+                "steps[?stepName=='publisher-signoff'].status | [0]",
+                response
+            )
+            if signoff_status == 'waitingForPublisherReview':
+                status = 'waitingForPublisherReview'
+
+        return status
 
     def get_operation(self, operation: str) -> dict:
         """
         Returns a dictionary status for the given operation.
         """
-        return get_cloud_partner_operation(self.access_token, operation)
+        endpoint = 'https://cloudpartner.azure.com{operation}'.format(
+            operation=operation
+        )
+
+        headers = get_cloud_partner_api_headers(self.access_token)
+        response = process_request(
+            endpoint,
+            headers
+        )
+
+        return response
 
     @property
     def blob_service_client(self):
@@ -648,7 +767,13 @@ class AzureImage(object):
         If credentials not set and a file is available attempt
         to load credentials json as dictionary.
         """
-        if not self._credentials and self._credentials_file:
+        if not self._credentials and not self._credentials_file:
+            raise AzureImgUtilsException(
+                'No credentials dictionary or credentials file provided. '
+                'Unable to authenticate with Azure.'
+            )
+
+        if not self._credentials:
             creds_file = os.path.expanduser(self._credentials_file)
 
             with open(creds_file, 'r') as json_file:
